/*
 *  main.c
 *  This file is part of Leafpad
 *
 *  Copyright (C) 2004 Tarot Osuji
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <sys/stat.h>
#include "readeasy.h"
#include "cfgfile.h"
#include "window.h"
#include "file.h"
#include "font.h"

static gint init_config_file();

static gint init_config_file()
{
	FILE *fp;
	char *file_ptr,*conf_dir;
	struct stat st;

	//常量TST_RC_DIR=“.tst”在dir.h中设置
	conf_dir =g_build_filename(g_get_home_dir(), "." PACKAGE, NULL);
	if (stat(conf_dir,&st))
		mkdir (conf_dir,00755);

	file_ptr = g_strdup_printf ("%s/."PACKAGE"/readeasy.conf",g_get_home_dir());
	if (!(fp = fopen (file_ptr,"wb"))) {
		g_free (file_ptr); 
		return  -1;
	}
	g_free (conf_dir);

	fprintf (fp,"[Config]\n");
	fprintf (fp,"\n#左边\nwindow_left=0\n");
	fprintf (fp,"\n#上边\nwindow_top=0\n");
	fprintf (fp,"\n#窗口宽度\nwindow_width=500\n");
	fprintf (fp,"\n#窗口高度\nwindow_height=420\n");
	
/*	fprintf (fp,"\n#\nwordwrap=1\n");
	fprintf (fp,"\n#\nlinenumbers=1\n");
	fprintf (fp,"\n#\nautoindent=1\n");
	fprintf (fp,"\n#\ncharset=1\n");
	fprintf (fp,"\n#\nAutoOpenNextFile=1\n");*/
	fprintf (fp,"\n#字体\nFont=Monospace 12\n");
//	fprintf (fp,"\n#文件\nFile=\n");
/*	
	
	fprintf (fp,"\n#字体\nFont=Monospace 12\n");
	fprintf (fp,"\n#自动滚屏\nAutoSpeed=0\n");
	fprintf (fp,"\n#显示文件栏\nViewFileBar=0\n");
	fprintf (fp,"\nAdvanced_Id=1\n");*/
	g_free (file_ptr);	

	fclose (fp);
	return 0;
}



static gint load_config_file(StructData *sd)
{
	gchar *file_ptr=NULL;
	gchar *buf;
	ConfigFile* config_file;
	file_ptr = g_strdup_printf ("%s/."PACKAGE"/readeasy.conf",g_get_home_dir());
	
	if(!g_file_test(file_ptr,G_FILE_TEST_EXISTS))
		init_config_file(); 
	
	if (!(config_file = cfg_file_open_file (file_ptr))) 
	{
		g_print("Can not open file:%s",file_ptr);
	}
		sd->conf.top =(cfg_file_read_int(config_file,"Config","window_top",&buf))?buf:0;
		sd->conf.left=(cfg_file_read_int(config_file,"Config","window_left",&buf))?buf:0;
		sd->conf.width =(cfg_file_read_int(config_file,"Config","window_width",&buf))?buf:600;
		sd->conf.height=(cfg_file_read_int(config_file,"Config","window_height",&buf))?buf:400;
		sd->conf.fontname=(cfg_file_read_string (config_file,"Config","Font",&buf))?buf:g_strdup("Monospace 12");
		sd->conf.filename=(cfg_file_read_string (config_file,"Config","File",&buf))?buf:g_strdup("");
/*		sd->conf.wordwrap=(cfg_file_read_boolean (config_file,"Config","wordwrap",&buf))?buf:FALSE;
		sd->conf.linenumbers=(cfg_file_read_boolean (config_file,"Config","linenumbers",&buf))?buf:FALSE;
		sd->conf.autoindent=(cfg_file_read_boolean (config_file,"Config","autoindent",&buf))?buf:FALSE;
		sd->conf.charset=(cfg_file_read_string(config_file,"Config","charset",&buf))?buf:NULL;
		sd->conf.auto_speed=(cfg_file_read_boolean (config_file,"Config","AutoSpeed",&buf))?buf:FALSE;
		sd->conf.auto_next_file=(cfg_file_read_boolean (config_file,"Config","AutoOpenNextFile",&buf))?buf:TRUE;*/
		sd->conf.view_file_bar=(cfg_file_read_boolean (config_file,"Config","ViewFileBar",&buf))?buf:FALSE;
	return 0;
}
//#ifndef _DEBUG
static gint save_config_file(StructData *sd)
{
	gchar *file_ptr=NULL;
	ConfigFile* config_file;
	gint width, height,top,left;
	gchar *fontname;

	gtk_window_get_size(GTK_WINDOW(sd->mainwin->window), &width, &height);
	gtk_window_get_position(GTK_WINDOW(sd->mainwin->window),&left,&top); 
//	g_print("x=%d,y=%d\n",x,y);
	fontname = get_font_name_from_widget(sd->mainwin->textview);
	
	file_ptr = g_strdup_printf ("%s/."PACKAGE"/readeasy.conf",g_get_home_dir());
	config_file = cfg_file_open_file (file_ptr);
	
	if (!config_file) 
	{
		g_print("warning: can't save config file - %s\n", file_ptr);
		return -1;
	}	
	cfg_file_write_int (config_file,"Config","window_top",top);
	cfg_file_write_int (config_file,"Config","window_left",left);
	cfg_file_write_int (config_file,"Config","window_width",width);
	cfg_file_write_int (config_file,"Config","window_height",height);
	cfg_file_write_string (config_file,"Config","Font",fontname);
	cfg_file_write_string (config_file,"Config","File",sd->conf.filename);
//	cfg_file_write_boolean(config_file,"Config","ViewFileBar",sd->conf.view_file_bar);
	
	cfg_file_write_file (config_file,file_ptr);
	cfg_file_free (config_file);
	g_free(file_ptr);

}

void real_dir()
{
	int pid;
	char *ptr,linkpath[1024],buf[1024];

	pid = getpid();
	sprintf (linkpath,"/proc/%d/exe",pid);
	readlink(linkpath,buf,1023);
//	printf ("file: %s\n",buf);

	ptr = buf + strlen(buf);
	
	while (*--ptr != '/');
	*ptr = '\0';

//	printf ("dir: %s\n",buf);
	chdir(buf);
}



static void create_new_process(gchar *filename)
{
//	StructData *sd = g_malloc(sizeof(StructData));
//	FileInfo *fi;
	sd = g_malloc(sizeof(StructData));
	
	load_config_file(sd);//self
	sd->mainwin = create_main_window();	//in window.h
	gtk_widget_show_all(sd->mainwin->window);
	
	set_text_font_by_name(sd->mainwin->textview, sd->conf.fontname);
	
	
//	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(m_folder),FALSE);
//	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(m_folder),sd->conf.view_file_bar);
//	gtk_signal_emit_by_name(G_OBJECT(m_folder),"toggled","");
	
//	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(m_folder),FALSE);
//	cb_view_file_activate(NULL,"button");

	if(sd->conf.filename!="")
	{
		file_open_real(sd->mainwin->textview,sd->conf.filename);
		set_main_window_title();
	}

	gtk_main();
	save_config_file(sd);
}
//#endif
gint main(gint argc, gchar *argv[])
{
	gchar *filename = NULL;
	gchar **strs;
	
//	bindtextdomain(PACKAGE, LOCALEDIR);
//	bind_textdomain_codeset(PACKAGE, "UTF-8");
//	textdomain(PACKAGE);
	real_dir();
	if(fork()) exit(0);
	setsid();
	gtk_init(&argc, &argv);
	
	if (argv[1]) {
		if (g_strstr_len(argv[1], 5, "file:")) {
			filename = g_filename_from_uri(argv[1], NULL, NULL);
			if (g_strrstr(filename, " ")) {
				strs = g_strsplit(filename, " ", -1);
				g_free(filename);
				filename = g_strjoinv("\\ ", strs);
				g_strfreev(strs);
			}
		} else {
			filename = g_strdup(argv[1]);
		}
	}
	
	create_new_process(filename);//self
	
	return 0;
}
